#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import os
import grpc
from rudder_autosearch.protos import common_pb2
from rudder_autosearch.protos import manager_service_pb2
from rudder_autosearch.protos import manager_service_pb2_grpc


class AMaasTools(object):
    """ AMaasTools: used to generate related output path and report to manager"""

    def __init__(self, job_id, trial_id):
        self._job_id = job_id
        self._trial_id = trial_id

    def _get_trial_dir_prefix(self):
        """ get $SHARED_OUTPUT_DIR/$trial_idx prefix"""
        shared_output_dir = os.environ.get("SHARED_OUTPUT_DIR")
        if not shared_output_dir:
            raise ValueError("env SHARED_OUTPUT_DIR is empty!")
        return os.path.join(shared_output_dir, self._trial_id)

    def get_checkpoint_dir(self):
        """ get checkpoint output dir for trainer
            required os env: $SHARED_OUTPUT_DIR, $TRIAL_ID
        """
        prefix = self._get_trial_dir_prefix()
        return os.path.join(prefix, "checkpoint")

    def get_log_dir(self):
        """ get log dir for current trial"""
        prefix = self._get_trial_dir_prefix()
        return os.path.join(prefix, "log")

    def get_model_export_dir(self):
        """ get model export dir"""
        prefix = self._get_trial_dir_prefix()
        return os.path.join(prefix, "export_model")

    def report_intermediate_result(self, metric, checkpoint_path=None):
        """ report intermediate metric to manager, required env $MANAGER_ADDRESS
        :param metric: json dict key/value
        :param checkpoint_path: predefined parameter, left empty here
        :return: request success/fail, should stop, error_msg
        """
        manager_addr = os.environ.get("MANAGER_ADDRESS")
        if not manager_addr:
            raise ValueError("env MANAGER_ADDRESS is empty")
        channel = grpc.insecure_channel(manager_addr)
        stub = manager_service_pb2_grpc.ManagerServiceStub(channel)
        request = manager_service_pb2.ReportMetricRequest(job_id=self._job_id,
                                                          trial_id=self._trial_id,
                                                          metric=json.dumps(metric))
        if checkpoint_path:
            request.checkpoint_path = checkpoint_path
        retry_times = 3
        error_msg = ''
        for i in range(retry_times):
            # add retry
            try:
                response = stub.OnReportIntermediateResult(request)
                if response.ret_code == common_pb2.SUCC:
                    return True, response.stop, ''
                else:
                    error_msg = response.ret_msg
                    break
            except grpc.RpcError as e:
                error_msg = str(e)
            except Exception as e:
                error_msg = str(e)
        return False, False, error_msg

    def report_final_result(self, metric, export_model_path, checkpoint_path=None):
        """ report intermediate metric to manager, required env $MANAGER_ADDRESS
        :param metric: json dict key/value
        :param checkpoint_path: checkpoint path, used by PBT search algorithm, only output weight
        :param export_model_path: model path, required by all trial, output weight and structure
        :return: request success/fail, error_msg
        """
        manager_addr = os.environ.get("MANAGER_ADDRESS")
        if not manager_addr:
            raise ValueError("env MANAGER_ADDRESS is empty")
        channel = grpc.insecure_channel(manager_addr)
        stub = manager_service_pb2_grpc.ManagerServiceStub(channel)
        request = manager_service_pb2.ReportMetricRequest(job_id=self._job_id,
                                                          trial_id=self._trial_id,
                                                          metric=json.dumps(metric),
                                                          export_model_path=export_model_path)
        if checkpoint_path:
            request.checkpoint_path = checkpoint_path
        retry_times = 3
        error_msg = ''
        for i in range(retry_times):
            # add retry
            try:
                response = stub.OnReportFinalResult(request)
                if response.ret_code == common_pb2.SUCC:
                    return True, ''
                else:
                    error_msg = response.ret_msg
                    break
            except grpc.RpcError as e:
                error_msg = str(e)
            except Exception as e:
                error_msg = str(e)
        return False, error_msg




